From: Yinghai Lu <yhlu.kernel@gmail.com>
To: Ingo Molnar <mingo@elte.hu>, Thomas Gleixner <tglx@linutronix.de>,
"H. Peter Anvin" <hpa@zytor.com>,
"Eric W. Biederman" <ebiederm@xmission.com>,
Dhaval Giani <dhaval@linux.vnet.ibm.com>,
Mike Travis <travis@sgi.com>,
Andrew Morton <akpm@linux-foundation.org>
Cc: linux-kernel@vger.kernel.org, Yinghai Lu <yhlu.kernel@gmail.com>
Subject: [PATCH 38/42] seperate irq_descX with irq_descX_free
Date: Fri, 8 Aug 2008 14:52:44 -0700 [thread overview]
Message-ID: <1218232368-31228-39-git-send-email-yhlu.kernel@gmail.com> (raw)
In-Reply-To: <1218232368-31228-38-git-send-email-yhlu.kernel@gmail.com>
so later don't need compare with -1U
Signed-off-by: Yinghai Lu <yhlu.kernel@gmail.com>
---
include/linux/irq.h | 2 +-
kernel/irq/handle.c | 113 ++++++++++++++++++++++++++------------------------
2 files changed, 60 insertions(+), 55 deletions(-)
diff --git a/include/linux/irq.h b/include/linux/irq.h
index 0d9578e..d130119 100644
--- a/include/linux/irq.h
+++ b/include/linux/irq.h
@@ -223,7 +223,7 @@ extern struct irq_desc *irq_descX;
extern struct irq_desc *irq_descX;
#define for_each_irq_desc(irqX, desc) \
- for (desc = irq_descX, irqX = desc->irq; desc && irqX != -1U; desc = desc->next, irqX = desc ? desc->irq: -1U)
+ for (desc = irq_descX, irqX = desc->irq; desc; desc = desc->next, irqX = desc ? desc->irq: -1U)
#endif
diff --git a/kernel/irq/handle.c b/kernel/irq/handle.c
index 194d815..0f5ea0c 100644
--- a/kernel/irq/handle.c
+++ b/kernel/irq/handle.c
@@ -111,6 +111,11 @@ static void init_kstat_irqs(struct irq_desc *desc, int nr_desc, int nr)
}
}
+#ifdef CONFIG_HAVE_SPARSE_IRQ
+static struct irq_desc *irq_descX_free;
+struct irq_desc *irq_descX;
+#endif
+
static void __init init_work(void *data)
{
struct dyn_array *da = data;
@@ -126,13 +131,16 @@ static void __init init_work(void *data)
#endif
}
+ /* init kstat_irqs, nr_cpu_ids is ready already */
+ init_kstat_irqs(desc, *da->nr, nr_cpu_ids);
+
#ifdef CONFIG_HAVE_SPARSE_IRQ
for (i = 1; i < *da->nr; i++)
desc[i-1].next = &desc[i];
-#endif
- /* init kstat_irqs, nr_cpu_ids is ready already */
- init_kstat_irqs(desc, *da->nr, nr_cpu_ids);
+ irq_descX_free = irq_descX;
+ irq_descX = NULL;
+#endif
}
#ifdef CONFIG_HAVE_SPARSE_IRQ
@@ -147,23 +155,17 @@ static int __init parse_nr_irq_desc(char *arg)
early_param("nr_irq_desc", parse_nr_irq_desc);
-struct irq_desc *irq_descX;
DEFINE_DYN_ARRAY(irq_descX, sizeof(struct irq_desc), nr_irq_desc, PAGE_SIZE, init_work);
struct irq_desc *irq_desc(unsigned int irq)
{
struct irq_desc *desc;
- BUG_ON(irq == -1U);
-
- desc = &irq_descX[0];
+ desc = irq_descX;
while (desc) {
if (desc->irq == irq)
return desc;
- if (desc->irq == -1U)
- return NULL;
-
desc = desc->next;
}
return NULL;
@@ -173,21 +175,12 @@ struct irq_desc *irq_desc_with_new(unsigned int irq)
struct irq_desc *desc, *desc_pri;
int i;
int count = 0;
- unsigned long phys;
- unsigned long total_bytes;
-
- BUG_ON(irq == -1U);
- desc_pri = desc = &irq_descX[0];
+ desc_pri = desc = irq_descX;
while (desc) {
if (desc->irq == irq)
return desc;
- if (desc->irq == -1U) {
- desc->irq = irq;
- printk("found new irq_desc for irq %d\n", desc->irq);
- return desc;
- }
desc_pri = desc;
desc = desc->next;
count++;
@@ -196,48 +189,60 @@ struct irq_desc *irq_desc_with_new(unsigned int irq)
/*
* we run out of pre-allocate ones, allocate more
*/
- printk(KERN_DEBUG "try to get more irq_desc %d\n", nr_irq_desc);
- {
- /* double check if some one mess up the list */
- struct irq_desc *desc;
- int count = 0;
-
- desc = &irq_descX[0];
- while (desc) {
- printk("found irq_desc for irq %d\n", desc->irq);
- if (desc->next)
- printk("found irq_desc for irq %d and next will be irq %d\n", desc->irq, desc->next->irq);
- desc = desc->next;
- count++;
- }
- printk("all preallocted %d\n", count);
- }
+ if (!irq_descX_free) {
+ unsigned long phys;
+ unsigned long total_bytes;
- total_bytes = sizeof(struct irq_desc) * nr_irq_desc;
- if (after_bootmem)
- desc = kzalloc(total_bytes, GFP_ATOMIC);
- else
- desc = __alloc_bootmem_nopanic(total_bytes, PAGE_SIZE, 0);
+ printk(KERN_DEBUG "try to get more irq_desc %d\n", nr_irq_desc);
- if (!desc)
- panic("please boot with nr_irq_desc= %d\n", count * 2);
+ total_bytes = sizeof(struct irq_desc) * nr_irq_desc;
+ if (after_bootmem)
+ desc = kzalloc(total_bytes, GFP_ATOMIC);
+ else
+ desc = __alloc_bootmem_nopanic(total_bytes, PAGE_SIZE, 0);
- phys = __pa(desc);
- printk(KERN_DEBUG "irq_desc ==> [%#lx - %#lx]\n", phys, phys + total_bytes);
+ if (!desc)
+ panic("please boot with nr_irq_desc= %d\n", count * 2);
- for (i = 0; i < nr_irq_desc; i++)
- init_one_irq_desc(&desc[i]);
+ phys = __pa(desc);
+ printk(KERN_DEBUG "irq_desc ==> [%#lx - %#lx]\n", phys, phys + total_bytes);
- for (i = 1; i < nr_irq_desc; i++)
- desc[i-1].next = &desc[i];
+ for (i = 0; i < nr_irq_desc; i++)
+ init_one_irq_desc(&desc[i]);
- /* init kstat_irqs, nr_cpu_ids is ready already */
- init_kstat_irqs(desc, nr_irq_desc, nr_cpu_ids);
+ for (i = 1; i < nr_irq_desc; i++)
+ desc[i-1].next = &desc[i];
- desc->irq = irq;
- desc_pri->next = desc;
- printk("1 found new irq_desc for irq %d and pri will be irq %d\n", desc->irq, desc_pri->irq);
+ /* init kstat_irqs, nr_cpu_ids is ready already */
+ init_kstat_irqs(desc, nr_irq_desc, nr_cpu_ids);
+
+ irq_descX_free = desc;
+ }
+ desc = irq_descX_free;
+ irq_descX_free = irq_descX_free->next;
+ desc->next = NULL;
+ if (desc_pri)
+ desc_pri->next = desc;
+ else
+ irq_descX = desc;
+ desc->irq = irq;
+ printk("found new irq_desc for irq %d\n", desc->irq);
+ {
+ /* dump the results */
+ struct irq_desc *desc;
+ unsigned long phys;
+ unsigned long bytes = sizeof(struct irq_desc);
+ unsigned int irqx;
+
+ printk(KERN_DEBUG "=========================== %d\n", irq);
+ printk(KERN_DEBUG "irq_desc dump after get that for %d\n", irq);
+ for_each_irq_desc(irqx, desc) {
+ phys = __pa(desc);
+ printk(KERN_DEBUG "irq_desc %d ==> [%#lx - %#lx]\n", irqx, phys, phys + bytes);
+ }
+ printk(KERN_DEBUG "===========================\n");
+ }
return desc;
}
#else
--
1.5.4.5
next prev parent reply other threads:[~2008-08-08 22:09 UTC|newest]
Thread overview: 66+ messages / expand[flat|nested] mbox.gz Atom feed top
2008-08-08 21:52 [PATCH 00/42] dyn_array/nr_irqs/sparse_irq support v5 Yinghai Lu
2008-08-08 21:52 ` [PATCH 01/42] 8250: Remove NR_IRQ usage Yinghai Lu
2008-08-08 21:52 ` [PATCH 02/42] x86: add after_bootmem for 32bit Yinghai Lu
2008-08-08 21:52 ` [PATCH 03/42] x86: remove irq_vectors_limits Yinghai Lu
2008-08-08 21:52 ` [PATCH 04/42] add dyn_array support Yinghai Lu
2008-08-08 21:52 ` [PATCH 05/42] add per_cpu_dyn_array support Yinghai Lu
2008-08-08 21:52 ` [PATCH 06/42] x86: alloc dyn_array all alltogether Yinghai Lu
2008-08-08 21:52 ` [PATCH 07/42] x86: enable dyn_array support Yinghai Lu
2008-08-08 21:52 ` [PATCH 08/42] introduce nr_irqs Yinghai Lu
2008-08-08 21:52 ` [PATCH 09/42] x86: using nr_irqs Yinghai Lu
2008-08-08 21:52 ` [PATCH 10/42] drivers/char to use nr_irqs Yinghai Lu
2008-08-08 21:52 ` [PATCH 11/42] drivers/net " Yinghai Lu
2008-08-08 21:52 ` [PATCH 12/42] drivers intr remapping " Yinghai Lu
2008-08-08 21:52 ` [PATCH 13/42] drivers/pcmcia " Yinghai Lu
2008-08-08 21:52 ` [PATCH 14/42] drivers/rtc " Yinghai Lu
2008-08-08 21:52 ` [PATCH 15/42] drivers/scsi " Yinghai Lu
2008-08-08 21:52 ` [PATCH 16/42] drivers/serial " Yinghai Lu
2008-08-08 21:52 ` [PATCH 17/42] drivers proc " Yinghai Lu
2008-08-08 21:52 ` [PATCH 18/42] drivers xen events " Yinghai Lu
2008-08-08 21:52 ` [PATCH 19/42] make irq_timer_state to use dyn_array Yinghai Lu
2008-08-08 21:52 ` [PATCH 20/42] make irq2_iommu " Yinghai Lu
2008-08-08 21:52 ` [PATCH 21/42] make irq_desc " Yinghai Lu
2008-08-08 21:52 ` [PATCH 22/42] irq: make irqs in kernel stat use per_cpu_dyn_array Yinghai Lu
2008-08-08 21:52 ` [PATCH 23/42] x86: use dyn_array in io_apic_xx.c Yinghai Lu
2008-08-08 21:52 ` [PATCH 24/42] x86: get mp_irqs from madt Yinghai Lu
2008-08-08 21:52 ` [PATCH 25/42] x86: remove nr_irq_vectors Yinghai Lu
2008-08-08 21:52 ` [PATCH 26/42] x86_64: use irq_desc() together with dyn_array Yinghai Lu
2008-08-08 21:52 ` [PATCH 27/42] x86: add irq_cfg in io_apic_64.c Yinghai Lu
2008-08-08 21:52 ` [PATCH 28/42] x86: put irq_2_pin pointer into irq_cfg Yinghai Lu
2008-08-08 21:52 ` [PATCH 29/42] x86: put timer_rand_state pointer into irq_desc Yinghai Lu
2008-08-08 21:52 ` [PATCH 30/42] x86: move kstat_irqs from kstat to irq_desc Yinghai Lu
2008-08-08 21:52 ` [PATCH 31/42] replace loop with nr_irqs with for_each_irq_desc Yinghai Lu
2008-08-08 21:52 ` [PATCH 32/42] replace loop with nr_irqs with for_each_irq_icfg Yinghai Lu
2008-08-08 21:52 ` [PATCH 33/42] remove >= nr_irqs checking with config_have_sparse_irq Yinghai Lu
2008-08-08 21:52 ` [PATCH 34/42] x86_64: add irq_desc in function in paramater Yinghai Lu
2008-08-08 21:52 ` [PATCH 35/42] x86: check with without_new in show_interrupts Yinghai Lu
2008-08-08 21:52 ` [PATCH 36/42] x86_64: introduce irq_cfg_with_new Yinghai Lu
2008-08-08 21:52 ` [PATCH 37/42] x86_64: introduce irq_desc_with_new Yinghai Lu
2008-08-08 21:52 ` Yinghai Lu [this message]
2008-08-08 21:52 ` [PATCH 39/42] x86_64: sperate irq_cfgx with irq_cfgx_free Yinghai Lu
2008-08-08 21:52 ` [PATCH 40/42] x86_64: make /proc/interrupts works with dyn irq_desc Yinghai Lu
2008-08-08 21:52 ` [PATCH 41/42] x86_64: remove one nr_irqs in show_stat Yinghai Lu
2008-08-08 21:52 ` [PATCH 42/42] x86: put irq_2_iommu pointer into irq_desc Yinghai Lu
2008-08-09 1:00 ` [PATCH 08/42] introduce nr_irqs Eric W. Biederman
2008-08-09 1:38 ` Yinghai Lu
2008-08-09 1:59 ` H. Peter Anvin
2008-08-09 7:30 ` Yinghai Lu
2008-08-09 16:02 ` Eric W. Biederman
2008-08-09 21:21 ` Yinghai Lu
2008-08-09 21:38 ` Eric W. Biederman
2008-08-09 22:35 ` Yinghai Lu
2008-08-09 6:07 ` Eric W. Biederman
2008-08-09 7:30 ` Yinghai Lu
2008-08-09 15:46 ` Eric W. Biederman
2008-08-09 21:37 ` Yinghai Lu
2008-08-08 22:38 ` [PATCH 01/42] 8250: Remove NR_IRQ usage Eric W. Biederman
2008-08-08 23:07 ` Yinghai Lu
2008-08-08 22:01 ` [PATCH 00/42] dyn_array/nr_irqs/sparse_irq support v5 H. Peter Anvin
2008-08-08 22:14 ` Yinghai Lu
2008-08-08 22:25 ` H. Peter Anvin
2008-08-08 22:30 ` Yinghai Lu
2008-08-08 22:33 ` H. Peter Anvin
2008-08-08 22:19 ` H. Peter Anvin
2008-08-08 22:26 ` Yinghai Lu
2008-08-08 23:40 ` Eric W. Biederman
2008-08-09 0:34 ` Yinghai Lu
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=1218232368-31228-39-git-send-email-yhlu.kernel@gmail.com \
--to=yhlu.kernel@gmail.com \
--cc=akpm@linux-foundation.org \
--cc=dhaval@linux.vnet.ibm.com \
--cc=ebiederm@xmission.com \
--cc=hpa@zytor.com \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@elte.hu \
--cc=tglx@linutronix.de \
--cc=travis@sgi.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox